#include <string>
#include <expected>
#include <iostream>
#include <cmath>

using namespace std;

enum class parse_error
{
    invalid_input,
    overflow
};

std::expected<double, parse_error> parse(const string& s) {
	std::size_t pos = 0; //liczba uzytych znakow
    double ret = std::stod(s, &pos);

    if (pos == 0)
        return std::unexpected(parse_error::invalid_input);
	else if (std::isinf(ret))
		return std::unexpected(parse_error::overflow);
    return ret;
}

int main()
{
	const auto data = {
		"12",
		"+13",
		"3.14",
		"inf",
		"meow",
	};

	for(auto s : data) {
		double d = parse(s).has_value_or(-1);
		cout << "s: " << s << ", "<< d << endl;
	}

	for(auto s : data) {
		cout << "s: " << s << ", "
		if(const auto d = parse(s); d.has_value() )
			cout << d << endl;
		else if ( d.error() == parse_error::invalid_input )
			cout << " error: invalid input" << endl;
		else if ( d.error() == parse_error::overflow )
			cout << " error: overflow" << endl;
		else
			cout << "unexpected!" << endl;
	}
	return 0;
}
